@@ -33,18 +33,10 @@
    is sizeof (int) * 3, and one extra for a possible -ve sign.  */
 #define ERRNO_DIGITS_MAX  (sizeof (int) * 3 + 1)
 
-/*
- * A limit on recursion, to avoid colliding with the heap. UEFI defines a baseline
- * stack size of 128 KiB. So, assuming at most 1-2 KiB per iteration this should
- * keep us safe.
- */
-#define MAX_RECURSION_DEPTH 64
-
 static unsigned long is_continue;
 static unsigned long active_loops;
 static unsigned long active_breaks;
 static unsigned long function_return;
-static unsigned long recursion_depth;
 
 #define GRUB_SCRIPT_SCOPE_MALLOCED      1
 #define GRUB_SCRIPT_SCOPE_ARGS_MALLOCED 2
@@ -562,8 +554,6 @@ gettext_append (struct grub_script_argv *result, const char *orig_str)
     if (*iptr == '$')
       dollar_cnt++;
   ctx.allowed_strings = grub_calloc (dollar_cnt, sizeof (ctx.allowed_strings[0]));
-  if (ctx.allowed_strings == NULL)
-    goto fail;
 
   if (parse_string (orig_str, gettext_save_allow, &ctx, 0))
     goto fail;
@@ -762,9 +752,6 @@ cleanup:
 	}
     }
 
-  if (result.args == NULL || result.argc == 0)
-    goto fail;
-
   if (! result.args[result.argc - 1])
     result.argc--;
 
@@ -801,7 +788,7 @@ cleanup:
 	      grub_free (expansions[j]);
 	    }
 	  grub_free (expansions);
-
+	  
 	  if (failed)
 	    {
 	      grub_script_argv_free (&unexpanded);
@@ -829,13 +816,7 @@ grub_script_execute_cmd (struct grub_script_cmd *cmd)
   if (cmd == 0)
     return 0;
 
-  recursion_depth++;
-
-  if (recursion_depth >= MAX_RECURSION_DEPTH)
-    return grub_error (GRUB_ERR_RECURSION_DEPTH, N_("maximum recursion depth exceeded"));
-
   ret = cmd->exec (cmd);
-  recursion_depth--;
 
   grub_snprintf (errnobuf, sizeof (errnobuf), "%d", ret);
   grub_env_set ("?", errnobuf);
@@ -916,10 +897,7 @@ grub_script_execute_sourcecode (const char *source)
 	  break;
 	}
 
-      /* Don't let trailing blank lines determine the return code. */
-      if (parsed_script->cmd)
-	ret = grub_script_execute (parsed_script);
-
+      ret = grub_script_execute (parsed_script);
       grub_script_free (parsed_script);
       grub_free (line);
     }
@@ -1017,9 +995,6 @@ grub_script_execute_cmdline (struct grub_script_cmd *cmd)
 	{
 	  /* As a last resort, try if it is an assignment.  */
 	  char *assign = grub_strdup (cmdname);
-          if (assign == NULL)
-            return grub_errno;
-
 	  char *eq = grub_strchr (assign, '=');
 
 	  if (eq)
